{
*****************************************************************************
*  Name    : USB kaitenkei Propeller                                         *
*  Author  : Hiroyuki Iizuka                                                *
*  Date    : 2010/03/22                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Program kaitenkeiPropeller

'---- configuration ---------------------------------------------------------
Device = 18F4550
Clock  = 48

Config
   USBDIV  = 1, 
   PLLDIV  = 5,
   CPUDIV  = OSC1_PLL2,
   FOSC    = HSPLL_HS,
   MCLRE   = OFF

'---- include file ----------------------------------------------------------
Include "Convert.bas"

'---- interrupt -------------------------------------------------------------
#option org_reset     = $2000
#option vector_isr_hi = $2008
#option vector_isr_lo = $2018

Const ipLow  = 1
Const ipHigh = 2

Const High_priority = 1
Const Low_priority  = 0

Dim Interrupt_HIGH_enable As INTCON.booleans(7)
Dim Interrupt_LOW_enable  As INTCON.booleans(6)
Dim Interrupt_priority    As RCON.booleans(7)

'---- 7segment LED const definitions ----------------------------------------
Const Digit_N  = $00   ' Blank
Const Digit_0  = $3F
Const Digit_1  = $06
Const Digit_2  = $5B
Const Digit_3  = $4F
Const Digit_4  = $66
Const Digit_5  = $6D
Const Digit_6  = $7D
Const Digit_7  = $07
Const Digit_8  = $7F
Const Digit_9  = $6F
Const Digit_DP = $80

Const LED_ON  = 1
Const LED_OFF = 0

Dim LED_DEV6     As PORTC.0
Dim LED_DEV5     As PORTC.1
Dim LED_DEV4     As PORTC.2
Dim LED_DEV3     As PORTE.0
Dim LED_DEV2     As PORTE.1
Dim LED_DEV1     As PORTE.2

Dim LED_SEG_PORT As PORTD
Dim LED_SEG_A    As PORTD.0
Dim LED_SEG_B    As PORTD.1
Dim LED_SEG_C    As PORTD.2
Dim LED_SEG_D    As PORTD.3
Dim LED_SEG_E    As PORTD.4
Dim LED_SEG_F    As PORTD.5
Dim LED_SEG_G    As PORTD.6

Dim led_val1       As Byte
Dim led_val2       As Byte
Dim led_val3       As Byte
Dim led_val4       As Byte
Dim led_val5       As Byte
Dim led_val6       As Byte
Dim led_pos        As Byte

'------------ HALL sensor ---------------------------------------------------
Dim PHOTO_REF_PORT As PORTB.0

'------------ 3mm LED -------------------------------------------------------
Dim LED1 As PORTB.5
Dim LED2 As PORTB.3
Dim LED3 As PORTA.2

'-------- INT0 --------------------------------------------------------------
Dim PortB0_changed         As INTCON.booleans(1)
Dim PortB0_int_enable      As INTCON.booleans(4)
Dim PortB0_int_rising_edge As INTCON2.booleans(6)

'-------- TMR0 --------------------------------------------------------------
Dim Timer0_overflowd             As INTCON.booleans(2)
Dim Timer0_interrupt_enable      As INTCON.booleans(5)
Dim Timer0_priority              As INTCON2.bits(2)
Dim Timer0_enable                As T0CON.booleans(7)
Dim Timer0_mode_8bit             As T0CON.booleans(6)
Dim Timer0_use_external_clock    As T0CON.booleans(5)
Dim Timer0_prescale_NOT_assign   As T0CON.booleans(3)
Dim Timer0_prescale_bit2         As T0CON.bits(2)
Dim Timer0_prescale_bit1         As T0CON.bits(1)
Dim Timer0_prescale_bit0         As T0CON.bits(0)

'-------- TMR1 --------------------------------------------------------------
Dim Timer1_overflowd             As PIR1.booleans(0) 
// Dim Timer1_peripheral_int_enable as INTCON.booleans(6)
Dim Timer1_interrupt_enable      As PIE1.booleans(0) 
Dim Timer1_prescale_bit0         As T1CON.bits(4)
Dim Timer1_prescale_bit1         As T1CON.bits(5)
Dim Timer1_use_external_clock    As T1CON.booleans(1)
Dim Timer1_enable                As T1CON.booleans(0)
Dim Timer1_priority              As IPR1.bits(0)

'-------- A/D converter -----------------------------------------------------
Dim ADconfig3 As ADCON1.3
Dim ADconfig2 As ADCON1.2
Dim ADconfig1 As ADCON1.1
Dim ADconfig0 As ADCON1.0

'-------- comparator --------------------------------------------------------
Dim Comparator_mode2 As CMCON.bits(2)
Dim Comparator_mode1 As CMCON.bits(1)
Dim Comparator_mode0 As CMCON.bits(0)

'-------- PORTB -------------------------------------------------------------
Dim PortB_pullups_disable As INTCON2.booleans(7)

'---- application [kaitenkei standard] --------------------------------------
Const CONST_MIN       = 720000000 ' TMR1 count 1 min @ 48MHz
Const ZERO_CLEAR_TIME = 1500      ' 1500 x 2ms = 3.0 sec
Const HALF_SECOND     = 91        ' timer1 overflow @ 0.5sec

Structure LWBstruc
  LongwordVal As LongWord Union  
  WordVal(2)  As Word     Union
  ByteVal(4)  As Byte     Union
End Structure 

Dim tempLWB As LWBstruc

Dim start_flag     As Boolean
Dim display_flag   As Boolean
Dim of_count       As Word     ' over flow counter

Dim TMR1L_value    As Byte
Dim TMR1H_value    As Byte
Dim TMR1_OVF_value As Word

Dim rpm            As LongWord
Dim tmr1_count     As LongWord
Dim count_div      As LongWord
Dim count_tmp1     As LongWord
Dim count_tmp2     As LongWord
Dim rotate         As LongWord
Dim rotate_value   As LongWord

Dim zero_display_counter As Word


'===============================================================================
' interrupt priority HIGH
'===============================================================================
Interrupt OnChangePORTB(ipHigh)
  '---- signal was detected
  If PortB0_changed Then
    If start_flag = true Then   ' save TMR1 value
      TMR1L_value = TMR1L
      TMR1H_value = TMR1H
      rotate = rotate + 1
      If of_count >= HALF_SECOND Then
        TMR1_OVF_value = of_count
        TMR1L = 0
        TMR1H = 0
        rotate_value = rotate
        of_count = 0
        rotate   = 0
        display_flag = true
      End If
    Else
      TMR1L = 0
      TMR1H = 0
      of_count = 0
      rotate   = 0
      start_flag = true
    End If
    zero_display_counter = 0
    PortB0_changed = false
  End If
End Interrupt


'===============================================================================
' interrupt priority LOW
'===============================================================================
Interrupt OnTimer(ipLow)

  '---- TMR1 overflow ----------
  If Timer1_overflowd Then
    Timer1_overflowd = false
    of_count = of_count + 1
  End If

  '--- TMR0 overflow (7SEG_LED storobe)   2 msec
  If Timer0_overflowd Then
    TMR0L = 69
    Select led_pos
      Case 1
        LED_DEV6 = LED_OFF
        LED_SEG_PORT = led_val1
        LED_DEV1 = LED_ON
        led_pos = led_pos + 1
      Case 2
        LED_DEV1 = LED_OFF
        LED_SEG_PORT = led_val2
        LED_DEV2 = LED_ON
        led_pos = led_pos + 1
      Case 3
        LED_DEV2 = LED_OFF
        LED_SEG_PORT = led_val3
        LED_DEV3 = LED_ON
        led_pos = led_pos + 1
      Case 4
        LED_DEV3 = LED_OFF
        LED_SEG_PORT = led_val4
        LED_DEV4 = LED_ON
        led_pos = led_pos + 1
      Case 5
        LED_DEV4 = LED_OFF
        LED_SEG_PORT = led_val5
        LED_DEV5 = LED_ON
        led_pos = led_pos + 1
      Case 6
        LED_DEV5 = LED_OFF
        LED_SEG_PORT = led_val6
        LED_DEV6 = LED_ON
        led_pos = 1
    EndSelect
    zero_display_counter = zero_display_counter + 1
    Timer0_overflowd = false   ' Clear interrupt flag
  End If
End Interrupt


'===============================================================================
' decode ascii-char -> 7segment-LED
'===============================================================================
Function led7seg_decode( ascii_char As Char ) As Byte
  Select ascii_char
    Case " "  result = Digit_N
    Case "0"  result = Digit_0
    Case "1"  result = Digit_1
    Case "2"  result = Digit_2
    Case "3"  result = Digit_3
    Case "4"  result = Digit_4
    Case "5"  result = Digit_5
    Case "6"  result = Digit_6
    Case "7"  result = Digit_7
    Case "8"  result = Digit_8
    Case "9"  result = Digit_9
    Else result = Digit_N
  EndSelect
End Function


'*******************************************************************************
' Program main
'*******************************************************************************
Dim led_txt  As String(6)

PortB_pullups_disable  = true

PortB0_int_enable      = true
PortB0_int_rising_edge = true

ADconfig0 = 1  ' PortA Digital I/O
ADconfig1 = 1
ADconfig2 = 1 
ADconfig3 = 1

Comparator_mode2 = 1 ' compatator off
Comparator_mode1 = 1 ' compatator off
Comparator_mode0 = 1 ' compatator off


Input(PHOTO_REF_PORT)

Output(LED_DEV6)
Output(LED_DEV5)
Output(LED_DEV4)
Output(LED_DEV3)
Output(LED_DEV2)
Output(LED_DEV1)
   
Output(LED_SEG_A)
Output(LED_SEG_B)
Output(LED_SEG_C)
Output(LED_SEG_D)
Output(LED_SEG_E)
Output(LED_SEG_F)
Output(LED_SEG_G)


LED_DEV6 = LED_OFF
LED_DEV5 = LED_OFF
LED_DEV4 = LED_OFF
LED_DEV3 = LED_OFF
LED_DEV2 = LED_OFF
LED_DEV1 = LED_OFF

led_val1 = Digit_8
led_val2 = Digit_8
led_val3 = Digit_8
led_val4 = Digit_8
led_val5 = Digit_8
led_val6 = Digit_8
led_pos  = 1  


'---- TMR0(8bitMODE) initialize --------------
Timer0_prescale_bit2 = 1   ' PreScaler 1:128
Timer0_prescale_bit1 = 1   ' PreScaler 1:128
Timer0_prescale_bit0 = 0   ' PreScaler 1:128
Timer0_prescale_NOT_assign = false
Timer0_mode_8bit           = true
Timer0_interrupt_enable    = true
Timer0_use_external_clock  = false
Timer0_enable              = true
Timer0_priority            = Low_priority

'---- TMR1(16bit) initialize -----------------
// Timer1_peripheral_int_enable = true
Timer1_interrupt_enable    = true 
Timer1_prescale_bit0       = 0
Timer1_prescale_bit1       = 0
Timer1_use_external_clock  = false
Timer1_enable              = true
Timer1_priority            = Low_priority

'---- interrpt -------------------------------
Enable(OnChangePORTB)
Enable(OnTimer)
Interrupt_priority    = true
Interrupt_HIGH_enable = true
Interrupt_LOW_enable  = true
 
rpm = 0
DelayMS(1000)
led_val1 = Digit_0
led_val2 = Digit_N
led_val3 = Digit_N
led_val4 = Digit_N
led_val5 = Digit_N
led_val6 = Digit_N

TMR1L_value    = 0
TMR1H_value    = 0
TMR1_OVF_value = 0
rotate_value   = 0
  
zero_display_counter = 0
start_flag   = false
display_flag = false
While( true )
  If zero_display_counter >= ZERO_CLEAR_TIME Then
    led_val1 = Digit_0
    led_val2 = Digit_N
    led_val3 = Digit_N
    led_val4 = Digit_N
    led_val5 = Digit_N
    led_val6 = Digit_N
    zero_display_counter = 0
    start_flag = false
  End If
  If display_flag Then
    tempLWB.ByteVal(0) = TMR1L_value
    tempLWB.ByteVal(1) = TMR1H_value
    tempLWB.WordVal(1) = TMR1_OVF_value
    tmr1_count = tempLWB.LongwordVal

    count_div  = CONST_MIN / tmr1_count
    count_tmp1 = count_div * rotate_value
    count_tmp2 = CONST_MIN Mod tmr1_count
    count_tmp2 = count_tmp2 * rotate_value
    count_tmp2 = count_tmp2 / tmr1_count
    rpm        = count_tmp1 + count_tmp2
    rpm        = rpm / 2
    
    led_txt = DecToStr( rpm, 6, " " )
    led_val1 = led7seg_decode( led_txt(5) )
    led_val2 = led7seg_decode( led_txt(4) )
    led_val3 = led7seg_decode( led_txt(3) )
    led_val4 = led7seg_decode( led_txt(2) )
    led_val5 = led7seg_decode( led_txt(1) )
    led_val6 = led7seg_decode( led_txt(0) )

    display_flag = false
  End If
Wend


'--------- E N D  O F  P R O G R A M -------------------------------------------
